#---------------------------------------------------------------------------
# Copyright IBM Corp. 2011
# All Rights Reserved
#---------------------------------------------------------------------------
################################################################################
# File name:    system/install/lib/PT/Install/ConfigItem/CreatePartition.pm
# Description: 	Create partition logic
# Author:       Joris Mihaeli
################################################################################

################################################################################
# Class name:   PT::Install::ConfigItem::CreatePartition
# Description: 	CreatePartition ConfigItem class 
# Base class:   PT::Install::ConfigItem
################################################################################
package PT::Install::ConfigItem::CreatePartition;

#pragmas
use strict;
use English;
use warnings;

use base qw(PT::Install::ConfigItem);


use PT::Install::BaseDefs;
use PT::Install::ParamsContainer;
use PT::System::Common;
use PT::System::Command;
use PT::Status;
use PT::Logger;

################################################################################
# Name:        new
# Description: constructor
# Params:      void
# Return:      reference to the created instance
################################################################################
sub new {
   my $proto = shift;
   my $class = ref($proto) || $proto;
   my $self = bless {}, $class;
   WriteLog(LOG_DEBUG, "Start new(@_)");
   $self = bless $self->SUPER::new(@_);
   $self->{paramContainer} = "";
   $self->{STATUS} = PT::Status->new();
   $self->{command} = PT::System::Command->new();
   return $self;
}
################################################################################
# Name:        init
# Description: initiator
# Params:      paramContainer with device and size
# Return:      status code
################################################################################
sub init {
   my $self = shift;
   $self->{paramContainer} = shift;
   $self->{STATUS} = $self->SUPER::init(@_);
   return $self->{STATUS} if (OK != $self->{STATUS});
   eval {
      my $deviceParam = ($self->{paramContainer})->getParam('device');
      if (! $deviceParam) {
         WriteLog(LOG_ERR, "Failed to get device parameter");
         $self->{STATUS}->errno(ERROR);
         $self->{STATUS}->message("Failed to get device parameter");
      }
      $self->{device} = $deviceParam->getValue();
      if (! $self->{device}) {
         $self->{STATUS}->errno(ERROR);
         $self->{STATUS}->message("Failed to get device value");
      }
      my $sizeParam = ($self->{paramContainer})->getParam('size');
      if (! $sizeParam) {
         WriteLog(LOG_WARN, "Failed to get size parameter");
         $self->{STATUS}->errno(ERROR);
         $self->{STATUS}->message("Failed to get size parameter");
      } else {
         $self->{size} = $sizeParam->getValue();
         if (! $self->{size}) {
            $self->{STATUS}->errno(ERROR);
            $self->{STATUS}->message("Failed to retrieve size value");
         }
      }
   };
   if ($EVAL_ERROR) {
      WriteLog(LOG_ERR, "Failed to init partition parameters $EVAL_ERROR");
      $self->{STATUS}->errno(ERROR);
      $self->{STATUS}->message("Failed to init partition parameters")
         if (! $self->{STATUS}->message);
   }
      
   return $self->{STATUS};
}

sub do
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "start do()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
   my $dev = $self->{device};
     
   $status = $self->_createPartition($self->{device}, $self->{size});
   if (OK ne $status) {
      WriteLog(LOG_ERR, "Failed to create partition");
   }  else  {
      WriteLog(LOG_INFO, "Successfully created partition $self->{device}");
   }
   return $status;

}

sub undo
{
   my $self = shift;
   WriteLog(LOG_DEBUG, "start undo()");
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
   my $partitionnum = _get_last_partition_number($self->{device});
   if ($partitionnum > 0) {
      $status = $self->_deletePartition($self->{device}, $partitionnum);
      if (OK ne $status) {
         WriteLog(LOG_ERR, "Failed to delete partition");
         return $status;
      }
   } else {
      WriteLog(LOG_ERR, "No partition number found");
   }
   WriteLog(LOG_DEBUG, "end undo()");
   return $status;
}

sub _createPartition {
   ###########
   # Function steps:
   # 1. Create partition table if no partitions have been created so far
   # 2. Check if the device has free area
   # 3. Determine the beginning of the device free area
   # 4. Calculate the end of the partition
   # 5. Create the partition
   ###########
   my $self = shift;
   my ($dev, $size) = @_;
   my $keepDev = $dev;
   $dev = "/dev/mapper/$dev" if ($dev !~ /^\/dev\/mapper\//);
   my $status = PT::Status->new();
   $status->errno(OK);   # so far
   my $command = PT::System::Command->new();
   #
   # create partition table
   #
   $keepDev .= "p";  # check for partitions
   my $cmd = "ls /dev/mapper | grep $keepDev | wc -l";
   $status = $command->run($cmd);
   if (OK != $status) {
      WriteLog (LOG_DEBUG, "Unable to check for partitions");
      status->errno(ERROR);
      return $status;
   }
   my $count = $command->stdout();
   chomp $count;
   #if this will be first partition on this device - create partition table
   if ($count == 0) { # partitions do not exist - create partition table
      WriteLog (LOG_INFO, "Creating partition table for $dev");
      $cmd = "parted -s $dev mklabel gpt";
      $status = $command->run($cmd);
      if (OK != $status) {
         WriteLog (LOG_ERR,
          "Failed to create partition table for $dev, rc $command->errno()");
         return $status;
      }
      WriteLog (LOG_INFO, "Partition table for $dev created successfully");
   }
   
   #
   # Assign partition size
   #
   WriteLog (LOG_INFO, "Assigning partition size for $dev");
   # check if the device has free area      
   $cmd = "parted -s $dev unit \'MiB\' print free | grep Free | wc -l";
   $status = $command->run($cmd);
   if (OK != $status) {
      WriteLog (LOG_ERR, "Failed to get free area");
      return $status;
   } else {
      my $count = $command->stdout();
      chomp($count);
      if ($count == 0) {
         WriteLog (LOG_DEBUG, "No free area on device $dev");
         $status->errno(ERROR);
         return $status;
      }
   }
   # 
   # Determine the device free area - start and end
   #
   
   # Find the start of the free area and ignore the space at the beginning of the LUN
   
   $cmd = "parted -s $dev unit \'MiB\' print free | grep Free | tail -1";
   # the output so far like: " 200000MiB  2287248MiB  2087248MiB  Free Space"
   $cmd .= " | awk '{print \$1}'";  # get the start of the free area
   $status = $command->run($cmd);
   my ($start, $end);
   if (OK != $status) {
      WriteLog (LOG_ERR, "Failed to get the start of the free area");
      return $status;
   } else {
      $start = $command->stdout();
      chomp($start);
      if ($start eq "") {  # check if we got the right data
         WriteLog (LOG_ERR, "Failed to get the start of the free area");
         $status->errno(ERROR);
         return $status;
      } elsif (substr($start, -3) eq "MiB") {   
         # the $start should be in 'MiB's - remove the units from the string
         chop($start);
         chop($start);
         chop($start);
      } else {
         WriteLog (LOG_ERR, "The start of the free area $start is not proper");
         status->errno(ERROR);
         return $status;
      }
   }
   
   # Find the end of the free area
   
   $end   = $start + $size;
   # have to add the unit chars again
   if ( $start < 1 ){
      $start = $start * 1024 ; 
      $start .= "KiB";
   } else {
      $start .= "MiB";
   }
   $end .= "MiB";
   #
   # Create partition
   #
   $cmd = "parted -s $dev mkpart primary $start $end";
     
   $status = $command->run($cmd);
   if (OK != $status) {
      WriteLog (LOG_ERR,
      "Failed to assign partition size of $size for $dev,rc $command->errno()");
   } else {
      WriteLog (LOG_INFO,
                "Partition size of $size for $dev assigned successfully");
   }
   return $status;
}


sub _deletePartition {
   my $self = shift;
   my ($dev, $partitionnum) = @_;
   $dev = "/dev/mapper/$dev" if ($dev !~ /^\/dev\/mapper\//);
   WriteLog (LOG_INFO, "Deleting partition $partitionnum for $dev");
   my $status = PT::Status->new();
   my $cmd = "parted -s $dev rm $partitionnum";
   my $command = PT::System::Command->new();
   $status = $command->run($cmd);
   if (OK != $status) {
      WriteLog (LOG_ERR,
      "Failed to delete partition $partitionnum from $dev, rc $command->errno()");
   }  else {
      WriteLog (LOG_INFO, "Partition deleted successfully");
   }
   return $status;
}

sub _get_last_partition_number{
   my $self = shift;
   my $dev = shift;
   $dev .= "p";	  # for partition distinction
   my $cmd = "ls /dev/mapper/ | grep $dev | wc -l";
   my $command = PT::System::Command->new();
   my $status = $command->run($cmd);
   my $count = $command->stdout();
   chomp $count;
   return $count;  
}

1;

